<?php

Paygine::checkCms();
class Paygine
{
	const MODULE_NAME = 'paygine';
	const DEFAULT_CURRENCY_CODE = 'RUB';
	const DEFAULT_CURRENCY_ISO = '643';
	const PAYMENT_TYPES = [
		'acquiring' => 'Стандартный эквайринг (одностадийная оплата)',
		'acquiring_two_step' => 'Стандартный эквайринг (двухстадийная оплата)',
		'halva' => 'Плайт (одностадийная оплата)',
		'halva_two_step' => 'Плайт (двухстадийная оплата)',
		'sbp' => 'Система Быстрых Платежей (СБП)',
		'loan' => 'Кредитование'
	];
	const TAX = [
		1 => 'ставка НДС 20%',
		2 => 'ставка НДС 10%',
		3 => 'ставка НДС расч. 20/120',
		4 => 'ставка НДС расч. 10/110',
		5 => 'ставка НДС 0%',
		6 => 'НДС не облагается'
	];
	const HASH_ALGO = [
		'md5' => 'MD5',
		'sha256' => 'SHA256'
	];
	const DEFAULT_TAX = 6;
	const FP_NAME_MAX_LENGTH = 128;
	const SHOP_CART_MAX_LENGTH = 1000;
	const ORDER_REGISTRATION_PATH = '/webapi/Register';
	const PAYMENT_TYPES_PATHS = [
		'acquiring' => '/webapi/Purchase',
		'acquiring_two_step' => '/webapi/Authorize',
		'halva' => '/webapi/custom/svkb/PurchaseWithInstallment',
		'halva_two_step' => '/webapi/custom/svkb/AuthorizeWithInstallment',
		'sbp' => '/webapi/PurchaseSBP',
		'loan' => '/webapi/custom/unicheckout/PurchaseWithLoanManager'
	];
	const OPERATIONS_TYPES = array(
		'PURCHASE',
		'PURCHASE_BY_QR',
		'AUTHORIZE',
		'REVERSE',
		'COMPLETE',
		'LOAN'
	);
	const OPERATION_APPROVED = 'APPROVED';
	const OPERATION_PURCHASE = 'PURCHASE';
	const OPERATION_PURCHASE_BY_QR = 'PURCHASE_BY_QR';
	const OPERATION_AUTHORIZE = 'AUTHORIZE';
	const OPERATION_COMPLETE = 'COMPLETE';
	const OPERATION_LOAN = 'LOAN';
	const OPERATION_REFUND = 'REVERSE';
	const ORDER_REGISTERED = 'REGISTERED';
	const ORDER_COMPLETED = 'COMPLETED';
	const ORDER_AUTHORIZED = 'AUTHORIZED';
	const ORDER_REFUNDED = 'CANCELED';
	const ORDER_BLOCKED = 'BLOCKED';
	const ORDER_EXPIRED = 'EXPIRED';
	const SHOP_ORDER_REGISTERED = 0;
	const SHOP_ORDER_IN_PROGRESS = 1;
	const SHOP_ORDER_CANCELED = 2;
	const SHOP_ORDER_COMPLETED = 3;
	const VSEGDADA = 'vda';
	const FINBOX = 'fb';
	
	public $sector;
	public $password;
	public $test;
	public $payment_type;
	public $tax;
	public $modal;
	public $hash_algo;
	public $order_registered;
	public $order_authorized;
	public $order_loan;
	public $order_completed;
	public $order_canceled;
	public $order_notify_customer;
	public $order_agreement;
	public $order_waiting_payment;
	
	function __construct(&$diafan, $payment_id)
	{
		$this->diafan = &$diafan;
		$pay_params = unserialize(DB::query_result("SELECT params FROM {payment} WHERE payment='%s' AND id=%d AND trash='0'", self::MODULE_NAME, $payment_id));
		foreach($pay_params as $name => $value){
			$short_name = str_replace(self::MODULE_NAME . '_', '', $name);
			$this->{$short_name} = $value;
		}
		if(empty($this->hash_algo))
			$this->hash_algo = 'md5';
	}
	
	static function checkCms()
	{
		if(!defined('DIAFAN')){
			$path = __FILE__;
			while(! file_exists($path . '/includes/404.php'))
			{
				$parent = dirname($path);
				if($parent == $path) exit;
				$path = $parent;
			}
			include $path . '/includes/404.php';
		}
	}
	
	public function getUrl($path = '')
	{
		$url = !empty($this->test) ? 'https://test.paygine.com' : 'https://pay.paygine.com';
		if($path)
			$url .= '/' . ltrim($path, '/');
		return $url;
	}
	
	static function centify($value)
	{
		return intval(strval(round($value, 2) * 100));
	}
	
	public function calcFiscalPositions($pay) {
		$shipping_price = !empty($pay['details']['delivery']['summ']) ? $pay['details']['delivery']['summ'] : 0;
		$tax = !empty($this->tax) ? $this->tax : self::DEFAULT_TAX;
		$fiscal_positions = '';
		$fiscal_amount = 0;
		foreach ($this->prepareOrderItems($pay) as $item) {
			$fiscal_positions .= $item['quantity'] . ';';
			$element_price = $this->centify($item['price']);
			$fiscal_positions .= $element_price . ';';
			$fiscal_positions .= $tax . ';';
			$fiscal_positions .= str_replace([';', '|'], '', mb_substr($item['text'], 0, self::FP_NAME_MAX_LENGTH)) . '|';
			$fiscal_amount += $item['quantity'] * $element_price;
		}
		
		if ($shipping_price > 0) {
			$fiscal_positions .= '1;';
			$shipping_price = round($shipping_price, 2);
			$fiscal_element_price = $this->centify($shipping_price);
			$fiscal_positions .= $fiscal_element_price . ';';
			$fiscal_positions .= $tax . ';';
			$fiscal_positions .= 'Delivery' . '|';
			$fiscal_amount += $fiscal_element_price;
		}
		
		$fiscalDiff = abs($fiscal_amount - $this->centify($pay['summ']));
		if ($fiscalDiff)
			$fiscal_positions .= '1;' . $fiscalDiff . ';6;' . 'Discount' . ';14|';
		
		return substr($fiscal_positions, 0, -1);
	}
	
	public function calcShopCart($pay) {
		$shipping_price = !empty($pay['details']['delivery']['summ']) ? $pay['details']['delivery']['summ'] : 0;
		$shop_cart = [];
		$sc_key = 0;
		$fiscal_amount = 0;
		foreach ($this->prepareOrderItems($pay) as $item) {
			$element_price = $this->centify($item['price']);
			$fiscal_amount += $item['quantity'] * $element_price;
			$shop_cart[$sc_key]['quantityGoods'] = (int) $item['quantity'];
			$shop_cart[$sc_key]['goodCost'] = $item['price'];
			$shop_cart[$sc_key]['name'] = $item['text'];
			$sc_key++;
		}
		
		if ($shipping_price > 0) {
			$shipping_price = round($shipping_price, 2);
			$fiscal_element_price = $this->centify($shipping_price);
			$shop_cart[$sc_key]['quantityGoods'] = 1;
			$shop_cart[$sc_key]['goodCost'] = $shipping_price;
			$shop_cart[$sc_key]['name'] = 'Delivery';
			$fiscal_amount += $fiscal_element_price;
		}
		$fiscalDiff = abs($fiscal_amount - $this->centify($pay['summ']));
		if ($fiscalDiff)
			return [];
		return $shop_cart;
	}
	
	public function sendRequest($url, $args, $method = 'POST', $repeat = true)
	{
		if($method === 'GET')
			$url .= '?' . http_build_query($args);
		$curl = curl_init();
		if(!$curl)
			throw new Exception('Failed to initialize new cURL session');
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		if($method === 'POST'){
			curl_setopt($curl, CURLOPT_POST, true);
			curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($args));
		}
		
		$response = false;
		while ($repeat) {
			$repeat--;
			$response = curl_exec($curl);
			if($response !== false)
				break;
			sleep(2);
		}
		curl_close($curl);
		return $response;
	}
	
	static function jsRedirect($url)
	{
		echo "<script>window.top.location.href = '{$url}'</script>";
		die();
	}
	
	public function getPcOperation($request)
	{
		$pc_order_id = (int) $request['id'];
		if(!$pc_order_id)
			throw new Exception('Failed to get PC order ID' . (!empty($request['error']) ? ' (error ' . $request['error'] . ')' : ''));
		$pc_operation_id = (int) $request['operation'];
		if(!$pc_operation_id)
			throw new Exception('Failed to get operation ID' . (!empty($request['error']) ? ' (error ' . $request['error'] . ')' : ''));
		
		$data = array(
			'sector' => $this->sector,
			'id' => $pc_order_id,
			'operation' => $pc_operation_id,
		);
		$this->signData($data, true);
		$response = $this->sendRequest($this->getUrl('/webapi/Operation'), $data);
		$pc_operation = $this->parseXML($response);
		if(!$pc_operation)
			throw new Exception('Unknown error');
		
		return $pc_operation;
	}
	
	/**
	 * @param array $response
	 * @return void
	 * @throws Exception
	 */
	public function responseValidation($response, $is_operation = true) {
		if($is_operation){
			if(!in_array($response['type'], self::OPERATIONS_TYPES))
				throw new Exception('Unknown operation type : ' . $response['type']);
			if(empty($response['reason_code']) && !empty($response['code']) && !empty($response['description']))
				throw new Exception("$response[description] (error code $response[code])");
		}
		if(empty($response['signature']))
			throw new Exception('Signature is empty');
		$xml_string = self::xmlArrayToString($response);
		$signature = $this->generateSign($xml_string, true);
		if ($signature !== $response['signature'])
			throw new Exception('Signature is not valid');
	}
	
	public function operationAmountValidation($pc_operation, $order_amount)
	{
		$operation_amount = $this->getOperationAmount($pc_operation);
		if (!$operation_amount || $operation_amount !== $this->centify($order_amount))
			return false;
		return true;
	}
	
	public function getOperationAmount($pc_operation)
	{
		return !empty($pc_operation['buyIdSumAmount']) ? (int) $pc_operation['buyIdSumAmount'] : (int) $pc_operation['amount'];
	}
	
	/**
	 * @param array $data
	 * @param bool $password
	 * @return void
	 */
	public function signData(array &$data, $password = false)
	{
		unset($data['signature']);
		$sign = implode('', $data);
		if($password)
			$sign .= $this->password;
		$data['signature'] = base64_encode(hash($this->hash_algo, $sign));
	}
	
	/**
	 * @param array|string $data
	 * @param string $password
	 * @return string
	 */
	public function generateSign($data, $password = '')
	{
		$string = is_array($data) ? implode('', $data) : (string) $data;
		if($password)
			$string .= $this->password;
		return base64_encode(hash($this->hash_algo, $string));
	}
	
	static function parseXML($response)
	{
		if (!$response)
			throw new Exception('Empty response');
		$xml = simplexml_load_string($response);
		if (!$xml)
			throw new Exception('Non valid XML was received');
		$valid_xml = json_decode(json_encode($xml), true);
		if (!$valid_xml)
			throw new Exception('Non valid XML was received');
		return $valid_xml;
	}
	
	public static function xmlArrayToString(array $array)
	{
		$res = '';
		foreach($array as $key => $value) {
			if($key === '@attributes' or $key === 'signature') continue;
			$res .= is_string($value) ? $value : self::xmlArrayToString($value);
		}
		return $res;
	}
	
	public static function orderStatusesList($empty_value = true)
	{
		$status_list = DB::query_fetch_key_value("SELECT id, [name] FROM {shop_order_status} WHERE trash='0'", "id", "name");
		return $empty_value ? array_merge([0 => ''], $status_list) : $status_list;
	}
	
	// заказы Кредитования комплитить только по LOAN
	public function getCustomOrderStatusId($operation)
	{
		switch($operation['type']){
			//case self::OPERATION_REGISTER:
			//	return !empty($this->order_registered) ? $this->order_registered : self::SHOP_ORDER_REGISTERED;
			case self::OPERATION_AUTHORIZE:
				if(!empty($operation['loan_type']))
					return !empty($this->order_loan) ? $this->order_loan : self::SHOP_ORDER_IN_PROGRESS;
				else
					return !empty($this->order_authorized) ? $this->order_authorized : self::SHOP_ORDER_IN_PROGRESS;
			case self::OPERATION_PURCHASE:
			case self::OPERATION_PURCHASE_BY_QR:
			case self::OPERATION_COMPLETE:
			case self::OPERATION_LOAN:
				// complete loan orders only by LOAN operation(not COMPLETE)
				if($operation['type'] === self::OPERATION_COMPLETE && !empty($operation['loan_type'])) break;
				return !empty($this->order_completed) ? $this->order_completed : self::SHOP_ORDER_IN_PROGRESS;
			case self::OPERATION_REFUND:
				return !empty($this->order_canceled) ? $this->order_canceled : self::SHOP_ORDER_CANCELED;
		}
		return false;
	}
	
	public function getPcPaymentType($operation)
	{
		switch($operation['type']) {
			case self::OPERATION_PURCHASE:
				return !empty($operation['buyIdSumAmount']) ? 'halva' : 'acquiring';
			case self::OPERATION_AUTHORIZE:
				if(!empty($operation['buyIdSumAmount'])){
					return 'halva_two_steps';
				}elseif(!empty($operation['loan_type'])){
					return 'loan';
				}else{
					return 'acquiring_two_steps';
				}
			case self::OPERATION_PURCHASE_BY_QR:
				return 'sbp';
		}
		return false;
	}
	
	/**
	 * Переписанный метод оплаты с возможностью менять статус оплаты
	 * \Payment_inc::success (modules/payment/payment.inc.php:213)
	 * Действия при успешной оплате
	 *
	 * @param array $pay данные платежа
	 * @param string $type тип операции: all - все действия, pay - оплата, redirect - редирект на страницу платежа
	 * @return void
	 */
	public function order_success($pay, $status_id)
	{
		$this->updateOrderStatus($pay['element_id'], $status_id);
		DB::query("UPDATE {payment_history} SET status='pay', created=%d WHERE id=%d", time(), $pay["id"]);
		$order_rew = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND trash='0' AND element_type='element' AND element_id IN (SELECT id FROM {site} WHERE module_name='%s' AND [act]='1' AND trash='0')", $pay['module_name']);
		$this->diafan->redirect('http'.(IS_HTTPS ? "s" : '').'://'.getenv("HTTP_HOST").'/'.(REVATIVE_PATH ? REVATIVE_PATH.'/' : '').$order_rew.'/step3/');
	}
	
	public function getPcOrderParamsByOrderId($order_id)
	{
		$res = DB::query_fetch_key_value("SELECT * FROM {%s} WHERE order_id=%d", self::MODULE_NAME, $order_id, 'key', 'value');
		if(!$res) return [];
		return ['order_id' => $order_id] + $res;
	}
	
	public function addPcOrderParams($order_id, $params)
	{
		foreach($params as $key => $value){
			DB::query("
			INSERT INTO {%s}
			(`order_id`, `key`, `value`)
			VALUES
			('%d', '%s', '%s')",
			self::MODULE_NAME, $order_id, $key, $value);
		}
	}
	
	public function updatePcOrderParams($order_id, $params)
	{
		foreach($params as $key => $value){
			$query = "
			UPDATE {%s}
			SET `value`='%s'
			WHERE `key`='%s'
			  AND `order_id`=%d LIMIT 1";
			DB::query($query,
			self::MODULE_NAME, $value, $key, $order_id);
		}
	}
	
	public function actionPcOrder($action, $pc_order)
	{
		try {
			$operation_type = self::getOperationType($action);
			if(!$operation_type)
				throw new Exception("Unknown action type ($action)");
			// order already processed
			if(
				($operation_type === self::OPERATION_COMPLETE && $pc_order['order_state'] === self::ORDER_COMPLETED)
				||
				($operation_type === self::OPERATION_REFUND && $pc_order['order_state'] === self::ORDER_REFUNDED)
			)
				throw new Exception("The order has been already $pc_order[status]");
			
			$order = DB::query_fetch_array("SELECT * FROM {shop_order} WHERE id=%d LIMIT 1", $pc_order['order_id']);
			if(!$order)
				throw new Exception("Can not found the order (ID: $pc_order[order_id])");
			$data = [
				'sector' => $this->sector,
				'id' => $pc_order['id'],
				// todo case if prepayment amount === 0 !
				'amount' => $pc_order['payment_type'] === 'loan' ? 0 : $this::centify($order['summ']),
				'currency' => $pc_order['currency']
			];
			$this->signData($data, true);
			
			$path = self::getActionPath($action, $pc_order['payment_type']);
			if(!$path)
				throw new Exception("Unknown operation path (action: $action, payment type: $pc_order[payment_type])");
			$response = $this->sendRequest($this->getUrl($path), $data);
			$pc_operation = $this->parseXML($response);
			$this->responseValidation($pc_operation);
			// There is no valid operation amount in case of LOAN (VsegdaDa\FinBox)
			if($pc_order['payment_type'] !== 'loan' && !$this->operationAmountValidation($pc_operation, $order['summ']))
				throw new Exception("Invalid operation amount");
		} catch(Exception $e) {
			Dev::exception($e, 'error');
			// redirect to order page
			$this->diafan->redirect('http'.(IS_HTTPS ? "s" : '').'://'.getenv("HTTP_HOST").'/'.(REVATIVE_PATH ? REVATIVE_PATH.'/' : '') . 'admin/order/edit' . $pc_order['order_id'] . '/');
		}
		// updating pc order state
		$this->updatePcOrderParams($pc_order['order_id'], ['order_state' => $pc_operation['order_state']]);
		// updating order status except COMPLETE operation for loan orders
		if($operation_type !== self::OPERATION_COMPLETE && $pc_order['payment_type'] !== 'loan')
			$this->updateOrderStatus($order['id'], $this->getCustomOrderStatusId($pc_operation));
		// redirect to order page
		$this->diafan->redirect('http'.(IS_HTTPS ? "s" : '').'://'.getenv("HTTP_HOST").'/'.(REVATIVE_PATH ? REVATIVE_PATH.'/' : '') . 'admin/order/edit' . $pc_order['order_id'] . '/');
	}
	
	public function getOperationType($action)
	{
		switch($action) {
			case 'complete': return Paygine::OPERATION_COMPLETE;
			case 'refund': return Paygine::OPERATION_REFUND;
		}
		return false;
	}
	
	public function getActionPath($action_type, $payment_type)
	{
		$prefix = ($payment_type == 'halva' || $payment_type == 'halva_two_steps') ? '/webapi/custom/svkb/' : '/webapi/';
		switch($action_type) {
			case 'complete': return $prefix . 'Complete';
			case 'refund': return $prefix . 'Reverse';
		}
		return false;
	}
	
	public function getPcOrderFromPc($pc_order_params)
	{
		$data = [
			'sector' => $this->sector,
			'id' => $pc_order_params['id'],
			'reference' => $pc_order_params['order_id']
		];
		$this->signData($data, true);
		$response = $this->sendRequest($this->getUrl('/webapi/Order'), $data);
		return $this->parseXML($response);
	}
	
	public function getOriginalPcOrderParams($original_pc_order)
	{
		$params = [];
		if(empty($original_pc_order['parameters']['parameter'])) return $params;
		foreach($original_pc_order['parameters']['parameter'] as $param) {
			$name = !empty($param['name']) ? $param['name'] : 'null';
			$i = 1;
			$key = $name;
			while(isset($params[$key])) {
				$key = $name . ++$i;
			}
			$params[$key] = !empty($param['value']) ? $param['value'] : 'null';
		}
		return $params;
	}
	
	public function getPaymentLoanType($order_id)
	{
		$pc_order_params = $this->getPcOrderParamsByOrderId($order_id);
		$original_pc_order = $this->getPcOrderFromPc($pc_order_params);
		$this->responseValidation($original_pc_order, false);
		$original_pc_order_params = $this->getOriginalPcOrderParams($original_pc_order);
		if(!empty($original_pc_order_params['vda_application_id']))
			return self::VSEGDADA;
		elseif(!empty($original_pc_order_params['fb_application_id']))
			return self::FINBOX;
		return false;
	}
	
	public function getNewOperationParams($pc_operation)
	{
		$new_order_params = [
			'order_state' => $pc_operation['order_state'],
			'amount' => $this->getOperationAmount($pc_operation)
		];
		$payment_type = $this->getPcPaymentType($pc_operation);
		if($payment_type)
			$new_order_params['payment_type'] = $payment_type;
		if(!empty($pc_operation['loan_type']))
			$new_order_params['loan_type'] = $pc_operation['loan_type'];
		return $new_order_params;
	}
	
	public function updateOrderStatus($order_id, $status_id)
	{
		$status = DB::query_fetch_array("SELECT * FROM {shop_order_status} WHERE id=%d LIMIT 1", $status_id);
		$order = DB::query_fetch_array("SELECT * FROM {shop_order} WHERE id=%d LIMIT 1", $order_id);
		$this->diafan->_order->set_status($order, $status);
	}
	
	public function  confirmOrder($order_id)
	{
		$pc_order_params = $this->getPcOrderParamsByOrderId($order_id);
		// the order already registered on PC
		if($pc_order_params) return false;
		$pay = DB::query_fetch_array("SELECT * FROM {payment_history} WHERE element_id=%d LIMIT 1", $order_id);
		if(!$pay) return false;
		$pay["details"] = $this->diafan->_order->details($pay['element_id']);
		$pay["desc"] = str_replace('%id', $pay['element_id'], $this->diafan->configmodules("desc_payment", 'shop'));
		$pg_order_id = $this->registerOrder($pay, true);//'notify_customer');
		if($pg_order_id){
			$this->updateOrderStatus($pay['element_id'], $this->order_waiting_payment);
			$this->addPcOrderParams($pay['element_id'], [
				'id' => $pg_order_id,
				'payment_id' => $pay['payment_id'],
				'payment_type' => '',
				'amount' => '',
				'currency' => self::DEFAULT_CURRENCY_ISO,
				'order_state' => self::ORDER_REGISTERED
			]);
		}
		
		
		// redirect to order page
		$this->diafan->redirect('http'.(IS_HTTPS ? "s" : '').'://'.getenv("HTTP_HOST").'/'.(REVATIVE_PATH ? REVATIVE_PATH.'/' : '') . 'admin/order/edit' . $order_id . '/');
	}
	
	public function registerOrder($pay, $notify_customer = false)
	{
		$order_amount = self::centify($pay['summ']);
		$currency = self::DEFAULT_CURRENCY_ISO;
		$register_signature = $this->generateSign($this->sector . $order_amount . $currency, true);
		$fiscal_positions = $this->calcFiscalPositions($pay);
		$data = [
			'sector' => $this->sector,
			'reference' => $pay['id'],
			'amount' => $order_amount,
			'fiscal_positions' => $fiscal_positions,
			'description' => 'Заказ #' . $pay['element_id'],
			'email' => !empty($pay['details']['email']) ? $pay['details']['email'] : '',
			'phone' => !empty($pay['details']['phone']) ? $pay['details']['phone'] : '',
			'currency' => $currency,
			'mode' => 1,
			'url' => BASE_PATH_HREF . 'payment/get/paygine',
			'signature' => $register_signature,
			'notify_customer' => (int) $notify_customer
		];
		
		$pg_order_id = $this->sendRequest($this->getUrl() . self::ORDER_REGISTRATION_PATH, $data);
		if(intval($pg_order_id) == 0)
			throw new Exception(str_replace("\n", ' ', $pg_order_id));

		return $pg_order_id;
	}
	
	function prepareOrderItems($pay)
	{
		$items = [];
		if(!empty($pay['details']['goods'])){
			foreach($pay['details']['goods'] as $goods_item){
				$items[] = [
					'text' => $goods_item['name'],
					'price' => $goods_item['price'],
					'quantity' => $goods_item['count']
				];
			}
		}
		if(!empty($pay['details']['additional'])){
			foreach($pay['details']['additional'] as $additional_item){
				$items[] = [
					'text' => $additional_item['name'],
					'price' => $additional_item['summ'],
					'quantity' => '1'
				];
			}
		}
		return $items;
	}
}
