<?php
/**
 * Обработка данных, полученных от системы Paygine
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    6.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2018 OOO «Диафан» (http://www.diafan.ru/)
 */

include_once 'payment.paygine.lib.php';

Paygine::checkCms();

if(!empty($_REQUEST['callback'])){
	
	try{
		$request = file_get_contents("php://input");
		$pc_operation = Paygine::parseXML($request);
		if(empty($pc_operation['reference']))
			throw new Exception("Undefined reference");
		$pay = DB::query_fetch_array("SELECT * FROM {payment_history} WHERE id=%d LIMIT 1", $pc_operation['reference']);
		if(!$pay)
			throw new Exception("Payment $request[reference] not found");
		$paygine = new Paygine($this, $pay['payment_id']);
		$paygine->responseValidation($pc_operation);
		// reference is payment_history id (not order id)
		$order_id = $pay['element_id'];
		// Invalid operation amount may be in case of LOAN (VsegdaDa\FinBox)
		if(!$paygine->operationAmountValidation($pc_operation, $pay['summ'])){
			// Checking credit payment by getting order params from PC
			$pc_operation['loan_type'] = $paygine->getPaymentLoanType($order_id);
			if(empty($pc_operation['loan_type']))
				throw new Exception("Invalid operation amount");
		}
	} catch(Exception $e) {
		Dev::exception($e, 'error');
		die($e->getMessage());
	}
	
	if($pc_operation['state'] !== Paygine::OPERATION_APPROVED)
		die('operation not approved');
	
	// updating order status
	$paygine->updateOrderStatus($order_id, $paygine->getCustomOrderStatusId($pc_operation));
	// pc order update
	$paygine->updatePcOrderParams($order_id, $paygine->getNewOperationParams($pc_operation));
	exit('ok');
	
} elseif(isset($_REQUEST['reference']) || isset($_REQUEST['id']) || isset($_REQUEST['operation'])) {
	
	if(empty($_REQUEST['reference']))
		throw new Exception('Failed to get reference');
	// reference is payment_history id (not order id)
	$pay = DB::query_fetch_array("SELECT * FROM {payment_history} WHERE id='%s' LIMIT 1", $_REQUEST['reference']);
	if(!$pay) {
		Custom::inc('includes/404.php');
	}
	$paygine = new Paygine($this, $pay['payment_id']);
	// js redirect from iframe to main window
	if(!empty($paygine->modal) && empty($_REQUEST['skip_redirect']))
		Paygine::jsRedirect(BASE_PATH_HREF . $_SERVER['REQUEST_URI'] . '&skip_redirect=1');
	
	try{
		$pay = $this->diafan->_payment->check_pay($pay['id'], Paygine::MODULE_NAME);
		$order_id = $pay['element_id'];
		
		$db_pg_order_id = (int) DB::query_result("SELECT `value` FROM {%s} WHERE `order_id`='%s' AND `key`='id' LIMIT 1", Paygine::MODULE_NAME, $order_id);
		if(!$db_pg_order_id || $db_pg_order_id !== (int) $_REQUEST['id'])
			throw new Exception('Invalid request data');
		
		if(!empty($_REQUEST['error']))
			throw new Exception('An error occurred while paying for the order. Request: ' . var_export($_REQUEST, true));
		$pc_operation = $paygine->getPcOperation($_REQUEST);
		
		$paygine->responseValidation($pc_operation);
		// Invalid operation amount may be in case of LOAN (VsegdaDa\FinBox)
		if($pc_operation['type'] === Paygine::OPERATION_AUTHORIZE && !$paygine->operationAmountValidation($pc_operation, $pay['summ'])){
			// Checking credit payment by getting order params from PC
			$pc_operation['loan_type'] = $paygine->getPaymentLoanType($order_id);
			if(empty($pc_operation['loan_type']))
				throw new Exception("Invalid operation amount");
		}
	} catch(Exception $e) {
		Dev::exception($e, 'error');
		$this->diafan->_payment->fail($pay);
	}
	if($pc_operation['state'] !== Paygine::OPERATION_APPROVED)
		$this->diafan->_payment->fail($pay);
	// pc order update
	$paygine->updatePcOrderParams($order_id, $paygine->getNewOperationParams($pc_operation));
	$paygine->order_success($pay, $paygine->getCustomOrderStatusId($pc_operation));
	
} elseif(isset($_GET['rewrite'])) {
	$args = explode('/', $_GET["rewrite"]);
	try{
		$order_id = (int) $args[1];
		if(!$order_id)
			throw new Exception('Undefined order ID');
		$action = (string) $args[2];
		if(!$action)
			throw new Exception('Undefined action');
		
		$pay = DB::query_fetch_array("SELECT * FROM {payment_history} WHERE element_id='%s' LIMIT 1", $order_id);
		$paygine = new Paygine($this, $pay['payment_id']);
		
		if($action === 'confirm')
			$paygine->confirmOrder($order_id);
		
		$pc_order_params = $paygine->getPcOrderParamsByOrderId($order_id);
		if(!$pc_order_params)
			throw new Exception('Can not find order params by order ID (' . $order_id . ')');
		
		$paygine->actionPcOrder($action, $pc_order_params);
	}catch(Exception $e) {
		Dev::exception($e, 'error');
		die($e->getMessage());
	}

}